/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpixmap.h>
#include <qevent.h>
#include <qtimer.h>
#include <qstyle.h>
#include <qfileinfo.h>

#include "message_box.h"
#include "chat.h"
#include "config_dialog.h"
#include "debug.h"
#include "modules.h"
#include "screenshot.h"
#include "chatbutton.h"

#define MODULE_SCREENSHOT_VERSION 0.3.1

ScreenShot* screenShot;

extern "C" int screenshot_init()
{
	kdebugf();
	screenShot = new ScreenShot();
	return 0;
}

extern "C" void screenshot_close()
{
	kdebugf();
	delete screenShot;
}

ScreenShot::ScreenShot(QWidget * parent, const char * name, WFlags f)
 : QWidget(parent, name, Qt::WStyle_Customize|Qt::WStyle_NoBorder)
{
	kdebugf();

	connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
	ChatList::ConstIterator it;
	for ( it = chat_manager->chats().begin(); it != chat_manager->chats().end(); it++ )
		handleCreatedChat(*it);

	buttonPressed = false;

	// Config Dialog
	ConfigDialog::registerSlotOnCreate(this, SLOT(onCreateConfig()));
	ConfigDialog::registerSlotOnClose(this, SLOT(onDestroyConfig()));

	QStringList opts = QStringList::fromStrList(QImageIO::outputFormats());
	ConfigDialog::addTab("ScreenShot", dataPath("kadu/modules/data/screenshot/camera.png"));
		ConfigDialog::addVRadioGroup("ScreenShot", "ScreenShot", QT_TRANSLATE_NOOP("@default", "File format"), "fileFormat", opts, opts, "PNG", QT_TRANSLATE_NOOP("@default", "Select image format, which screenshots will be stored and sent in."));
		ConfigDialog::addSpinBox("ScreenShot", "ScreenShot", "Screenshots quality", "quality", -1, 100, 1, -1, QT_TRANSLATE_NOOP("@default", "-1 for autodetection, 0 for best file compression, 100 for best image quality."));
		ConfigDialog::addLineEdit("ScreenShot", "ScreenShot", QT_TRANSLATE_NOOP("@default", "Screenshots directory path"), "path", ggPath("images/"));
		ConfigDialog::addLineEdit("ScreenShot", "ScreenShot", QT_TRANSLATE_NOOP("@default", "Screenshots filename prefix"), "filenamePrefix", "shot");
}


ScreenShot::~ScreenShot()
{
	kdebugf();

	ConfigDialog::removeControl("ScreenShot", "Screenshots filename prefix");
	ConfigDialog::removeControl("ScreenShot", "Screenshots directory path");
	ConfigDialog::removeControl("ScreenShot", "Screenshots quality");
	ConfigDialog::removeControl("ScreenShot", "File format");
	ConfigDialog::removeTab("ScreenShot");

	ConfigDialog::unregisterSlotOnCreate(this, SLOT(onCreateConfig()));
	ConfigDialog::unregisterSlotOnClose(this, SLOT(onDestroyConfig()));

	disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	disconnect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
	ChatList::ConstIterator it;
	for ( it = chat_manager->chats().begin(); it != chat_manager->chats().end(); it++ )
		handleDestroyingChat(*it);
}

void ScreenShot::handleCreatedChat(Chat* chat)
{
	kdebugf();

	ChatButton* ss = new ChatButton(chat);
	ss->setPixmap(icons_manager.loadIcon(dataPath("kadu/modules/data/screenshot/camera_small.png")));
	ss->show();
	QToolTip::add(ss, tr("Send ScreenShot"));
	chatButtons[chat] = ss;
	connect(ss, SIGNAL(clicked(Chat*)), this, SLOT(takeShot(Chat*)));
}

void ScreenShot::handleDestroyingChat(Chat* chat)
{
	kdebugf();

	disconnect(chatButtons[chat], SIGNAL(clicked(Chat*)), this, SLOT(takeShot(Chat*)));
	delete chatButtons[chat];
	chatButtons.remove(chat);
}

void ScreenShot::chatCreated(const UinsList& ul)
{
	kdebugf();
	Chat* chat = chat_manager->findChatByUins(ul);
	handleCreatedChat(chat);
}

void ScreenShot::chatDestroying(const UinsList& ul)
{
	kdebugf();
	Chat* chat = chat_manager->findChatByUins(ul);
	handleDestroyingChat(chat);
}

void ScreenShot::mousePressEvent(QMouseEvent* e)
{
	kdebugf();
	if (e->button() == Qt::LeftButton)
	{
		region = QRect(e->pos(), e->pos());
		buttonPressed = true;
	}
}

void ScreenShot::mouseReleaseEvent(QMouseEvent* e)
{
	kdebugf();

	if (!buttonPressed)
		return;

	// Uwalnianie myszki, klawiatury
	buttonPressed = false;
	releaseMouse();
	releaseKeyboard();
	drawRegionRect();

	// Normalizowanie prostokta do zrzutu
	region.setBottomRight(e->pos());
	region.normalize();

	// Plik do zapisu:
	QString path = QDir::cleanDirPath(
			    config_file.readEntry("ScreenShot", "path", ggPath("images")) + "/" +
				config_file.readEntry("ScreenShot", "filenamePrefix", "shot") +
				QString::number(QDateTime::currentDateTime().toTime_t()) + "." +
				config_file.readEntry("ScreenShot", "fileFormat", "PNG").lower()
		);

	// Zrzut
	QPixmap shot = QPixmap::grabWindow(winId(), region.x(), region.y(), region.width(), region.height());
	handleShot(shot, path);
}

void ScreenShot::handleShot(QPixmap p, QString path)
{
	bool ret = p.save(
			path, config_file.readEntry("ScreenShot", "fileFormat", "PNG"),
			config_file.readNumEntry("ScreenShot", "quality", -1)
		);

	// Chowanie widgeta zrzutu i przywrcenie kursora.
	hide();
	QApplication::restoreOverrideCursor();

	if (!ret)
	{
		printf("Can't write file %s.\nAccess denied or other problem.", path.local8Bit().data());
		return;
	}

	if (!tempChat)
	{
		printf("ScreenShot module can't determinate Chat window. It should not happen! It's a bug!\n Please report it at Kadu Forum or to module author.\n");
		return;
	}

	QFileInfo f(path);
	int size = f.size();

	if (size == 0)
	{
		printf("File %s has 0 size!\nIt should be bigger.", path.local8Bit().data());
		return;
	}

	// Sprawdzanie rozmiaru zrzutu wobec rozmwcw
	UinsList uins = tempChat->uins();
	if (uins.count() > 1)
	{
		QStringList list;
		for ( uint i = 0; i < uins.count(); i++ )
		{
			if ( ( userlist.byUinValue(uins[i]).maxImageSize() * 1024) < size)
				list.append( userlist.byUinValue(uins[i]).altNick()) ;
		}
		if (list.count() > 0)
		{
			if (list.count() == uins.count())
			{
				MessageBox::wrn(
						tr("Image size is bigger than maximal image size\nset by all of conference contacts."),
						TRUE
					);
			}
			else
			{
				MessageBox::wrn(
						tr("Image size is bigger than maximal image size\nset by some of conference contacts:\n%1.")
							.arg(list.join(", ")),
						TRUE
					);
			}
		}
	}
	else
	{
		if ( ( userlist.byUinValue(uins[0]).maxImageSize() * 1024) < size)
		{
			MessageBox::wrn(
					tr("Image size is bigger than maximal image size set by %1.")
						.arg( userlist.byUinValue(uins[0]).altNick() ),
					TRUE
				);
			return;
		}
	}

	// Wklejanie [IMAGE] do okna Chat
	int x, y;
	tempChat->edit->getCursorPosition(&y, &x);
	tempChat->edit->insertAt(QString("[IMAGE ") + path + "]", y, x);
	tempChat->edit->moveCursor(QTextEdit::MoveEnd, FALSE);
	tempChat = NULL;
}

void ScreenShot::keyPressEvent(QKeyEvent* e)
{
	kdebugf();
	if (e->key() == Qt::Key_Escape)
	{
		releaseMouse();
		releaseKeyboard();
		hide();
	}
}

void ScreenShot::takeShot(Chat* chat)
{
	kdebugf();
	tempChat = chat;
	pixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
	resize(pixmap.size());
	setPaletteBackgroundPixmap(pixmap);
	showFullScreen();
	setCursor(crossCursor);
	QTimer::singleShot(100, this, SLOT(grabMouseSlot()));
}

void ScreenShot::grabMouseSlot()
{
	kdebugf();
	grabMouse();
	grabKeyboard();
}

void ScreenShot::mouseMoveEvent(QMouseEvent* e)
{
	kdebugf();
	if (!buttonPressed)
		return;

	drawRegionRect();
	region.setBottomRight(e->pos());
	drawRegionRect();
}

void ScreenShot::drawRegionRect()
{
	QPainter painter;
	painter.begin( this );
	painter.setRasterOp( NotROP );
	painter.setPen( QPen( color0, 1 ) );
	painter.setBrush( NoBrush );
	style().drawPrimitive(
			QStyle::PE_FocusRect,
			&painter,
			region,
			colorGroup(),
			QStyle::Style_Default,
			QStyleOption(colorGroup().base())
		);

	painter.end();
}

void ScreenShot::onCreateConfig()
{
	kdebugf();
	modules_manager->moduleIncUsageCount("screenshot");
}

void ScreenShot::onDestroyConfig()
{
	kdebugf();
	modules_manager->moduleDecUsageCount("screenshot");
}
